use specs::{saveload::MarkedBuilder, Builder, Entity, World, WorldExt};

use crate::{
    components::{Fleet, MeetingPointOwned, PlayerOwned},
    misc::{Persistence, WorldPersistence},
};

use super::Error;

#[derive(Default, Clone)]
pub struct FleetBuilder {
    player: Option<Entity>,
    meeting_point: Option<Entity>,
}

impl FleetBuilder {
    pub fn build(&self, world: &mut World) -> Result<Entity, Error> {
        let player = self.player.ok_or(Error::MissingValue("player"))?;
        let meeting_point = self
            .meeting_point
            .ok_or(Error::MissingValue("meeting_point"))?;

        let player_owned = PlayerOwned::new(player);
        let meeting_point_owned = MeetingPointOwned::new(meeting_point);
        let fleet = Fleet::new();

        let entity = world
            .create_entity()
            .marked::<<WorldPersistence as Persistence>::Marker>()
            .with(meeting_point_owned)
            .with(player_owned)
            .with(fleet)
            .build();

        Ok(entity)
    }

    pub fn player(mut self, player: Entity) -> Self {
        self.player = Some(player);

        self
    }

    pub fn meeting_point(mut self, meeting_point: Entity) -> Self {
        self.meeting_point = Some(meeting_point);

        self
    }
}
