// TODO
#![allow(dead_code)]
#![allow(unused_mut)]
#![allow(unused_imports)]
#![allow(unused_variables)]

use shrev::{EventChannel, ReaderId};
use specs::{
    prelude::*, Entities, Entity, LazyUpdate, ReadExpect, ReadStorage, System, World, WriteStorage,
};

use crate::{
    components::{
        Fleet, FleetOwned, MeetingPoint, MeetingPointOwned, Player, PlayerOwned, Ship, ShipCount,
    },
    misc::WorldHelper,
    Error,
};

pub struct FleetControl {
    fleet_control_event_id: ReaderId<FleetControlEvent>,
}

#[derive(Debug)]
pub enum FleetControlEvent {
    MoveToMeetingPoint {
        player: Entity,
        fleet: Entity,
        count: ShipCount,
        target: Entity,
    },
}

#[derive(SystemData)]
pub struct FleetControlData<'a> {
    lazy: Read<'a, LazyUpdate>,
    entities: Entities<'a>,
    fleet_control_events: ReadExpect<'a, EventChannel<FleetControlEvent>>,
    player_owned: WriteStorage<'a, PlayerOwned>,
    meeting_point_owned: WriteStorage<'a, MeetingPointOwned>,
    fleet_owned: WriteStorage<'a, FleetOwned>,
    meeting_points: WriteStorage<'a, MeetingPoint>,
    fleets: WriteStorage<'a, Fleet>,
    players: ReadStorage<'a, Player>,
    ships: ReadStorage<'a, Ship>,
}

impl FleetControl {
    pub fn new(world: &mut World) -> Result<Self, Error> {
        world.insert(EventChannel::<FleetControlEvent>::default());

        let fleet_control_event_id = world.register_event_reader::<FleetControlEvent>()?;

        Ok(Self {
            fleet_control_event_id,
        })
    }
}

impl<'a> System<'a> for FleetControl {
    type SystemData = FleetControlData<'a>;

    fn run(&mut self, data: Self::SystemData) {
        let FleetControlData {
            lazy,
            entities,
            fleet_control_events,
            mut player_owned,
            mut meeting_point_owned,
            mut fleet_owned,
            mut meeting_points,
            mut fleets,
            players,
            ships,
        } = data;

        /* TODO
        let events = fleet_control_events.read(&mut self.fleet_control_event_id);
        for event in events {
            match event {
                FleetControlEvent::Move(args) if args.count.is_all() => {
                    let orbit_owned = continue_if_none!(meeting_point_owned.get_mut(args.fleet));
                    orbit_owned.set_owner(args.target);
                }
                FleetControlEvent::Move(args) => {
                    let target_orbit = continue_if_none!(meeting_points.get_mut(args.target));
                    let player = continue_if_none!(players.get(args.player));
                    let mut count = args.count;

                    let target_fleet = match target_orbit.fleet_mut(player.index()) {
                        Some(fleet) => *fleet,
                        f @ None => {
                            let fleet = lazy
                                .create_entity(&entities)
                                .marked::<<WorldPersistence as Persistence>::Marker>()
                                .build();

                            player_owned
                                .insert(fleet, PlayerOwned::new(args.player))
                                .error("Unable to insert component: PlayerOwned");
                            meeting_point_owned
                                .insert(fleet, MeetingPointOwned::new(args.target))
                                .error("Unable to insert component: MeetingPointOwned");
                            fleets
                                .insert(fleet, Fleet::default())
                                .error("Unable to insert component: Fleet");

                            *f = Some(fleet);

                            fleet
                        }
                    };

                    let source_fleet = continue_if_none!(fleets.get(args.fleet));
                    let data = (&mut fleet_owned, &ships, source_fleet.owned());
                    for (fleet_owned, ship, _) in data.join() {
                        match ship.type_() {
                            ShipType::Fighter if count.fighter > 0 => {
                                count.fighter -= 1;
                                fleet_owned.set_owner(target_fleet);
                            }
                            ShipType::Bomber if count.bomber > 0 => {
                                count.bomber -= 1;
                                fleet_owned.set_owner(target_fleet);
                            }
                            ShipType::Transporter if count.transporter > 0 => {
                                count.transporter -= 1;
                                fleet_owned.set_owner(target_fleet);
                            }
                            _ => (),
                        }
                    }
                }
            }
        }
        */
    }
}
